@@ -42,11 +42,19 @@ typedef struct _GnomePrintMeta GnomePrintMeta;
 
 GType gnome_print_meta_get_type (void);
 
-GnomePrintContext * gnome_print_meta_new (void);
+GnomePrintContext * gnome_print_meta_new   (void);
+void                gnome_print_meta_reset (GnomePrintMeta *meta);
 
-gint           gnome_print_meta_get_length (const GnomePrintMeta *meta);
-const guchar * gnome_print_meta_get_buffer (const GnomePrintMeta *meta);
-int            gnome_print_meta_get_pages  (const GnomePrintMeta *meta);
+gint           gnome_print_meta_get_length    (const GnomePrintMeta *meta);
+const guchar * gnome_print_meta_get_buffer    (const GnomePrintMeta *meta);
+int            gnome_print_meta_get_pages     (const GnomePrintMeta *meta);
+gint           gnome_print_meta_get_page_name (const GnomePrintMeta *meta, guint page, guchar **page_name);
+
+gint gnome_print_meta_render      (const GnomePrintMeta *meta,
+				   GnomePrintContext *ctx);
+gint gnome_print_meta_render_page (const GnomePrintMeta *meta,
+				   GnomePrintContext *ctx, gint page,
+				   gboolean pageops);
 
 gint           gnome_print_meta_render_data      (GnomePrintContext *ctx, const guchar *data, gint length);
 gint           gnome_print_meta_render_data_page (GnomePrintContext *ctx, const guchar *data, gint length, gint page, gboolean pageops);
