@@ -25,6 +25,7 @@
  */
 
 #define PANGO_ENABLE_BACKEND	/* Needed to access PangoFcFont.font_pattern */
+/* behdad: Humm, font_pattern can be accessed using "pattern" property. */
 
 #include <config.h>
 
@@ -35,6 +36,12 @@
 #include <pango/pangofc-font.h>
 #include <pango/pangoft2.h>
 
+#ifdef PANGO_GLYPH_EMPTY
+#define EMPTY_GLYPH PANGO_GLYPH_EMPTY
+#else
+#define EMPTY_GLYPH 0
+#endif
+
 /**
  * is_gnome_print_key :
  * 
@@ -298,7 +305,7 @@ gnome_print_pango_glyph_string (GnomePrintContext *gpc, PangoFont *font, PangoGl
 	for (i = 0; i < glyphs->num_glyphs; i++) {
 		PangoGlyphInfo *gi = &glyphs->glyphs[i];
 
-		if (gi->glyph) {
+		if (gi->glyph != EMPTY_GLYPH) {
 			int x = x_off + gi->geometry.x_offset;
 			int y = gi->geometry.y_offset;
 			
