@@ -11,12 +11,11 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <popt.h>
 #include <sys/stat.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <fcntl.h>
-#include <ctype.h>
 #include <unistd.h>
 #include <sys/types.h>
 
@@ -24,11 +23,14 @@
 #include <sys/mman.h>
 #endif
 
-static gint debug;
+static gint debug = 0;
+static gchar **args = NULL;
 
-static struct poptOption options[] = {
-	{ "debug",   '\0', POPT_ARG_INT, &debug, 0,
+static const GOptionEntry options[] = {
+	{ "debug",   '\0', 0, G_OPTION_ARG_INT, &debug,
 	  "debug level",          "0,1,2,3"},
+	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &args,
+	  NULL,	NULL},
 	{ NULL }
 };
 
@@ -520,18 +522,22 @@ my_read_file (gchar *filename, gint *size)
 }
 
 int
-main (int argc, const char * argv[])
+main (int argc, char * argv[])
 {
-	poptContext popt;
-	const gchar **args;
+	GOptionContext *goption_context;
+	GError *error = NULL;
 	gchar *in_filename;
 	guchar *in;
 	gint size;
 
 	/* Args */
-	popt = poptGetContext ("checkxref", argc, argv, options, 0);
-	poptGetNextOpt (popt);
-	args = poptGetArgs (popt);
+	goption_context = g_option_context_new ("INPUT_FILE");
+	g_option_context_add_main_entries (goption_context, options, NULL);
+	g_option_context_parse (goption_context, &argc, &argv, &error);
+	g_option_context_free (goption_context);
+
+	if (error != NULL)
+		my_error ("%s", error->message);
 	if (!args || !args[0])
 		my_error ("Input file not specified");
 	if (debug)
